function [hest, flag, iter, resvec] = ...
    gradient_conjugue(funQh, par, b, tol, maxiter)
%
% Minimisation d'une forme quadratique par gradient conjugue linaire 
%
%    Copyright (C) 2012-2014  Yves Goussard
%
%    This program is free software: you can redistribute it and/or modify
%    it under the terms of the GNU General Public License as published by
%    the Free Software Foundation, either version 3 of the License, or
%    (at your option) any later version.
%
%    This program is distributed in the hope that it will be useful,
%    but WITHOUT ANY WARRANTY; without even the implied warranty of
%    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%    GNU General Public License for more details.
%
%    You should have received a copy of the GNU General Public License
%    along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
% Paramtres d'entre
%   funQh : nom d'une fonction fonction qui, pour tout vecteur
%           h, retourne le produit Q h, o Q est la matrice de la
%           forme quadratique  minimiser
%   par : structure contenant les paramtres utiliss par calcQh
%   b : vecteur b de la forme quadratique  minimiser
%   tol : seuil d'arrt portant sur la norme relative du gradient de
%         la forme quadratique  minimiser (|| Qh-b || / || b ||)
%   maxiter : nombre maximum d'itrations  effectuer
%
% Paramtres de sortie
%   hest : valeur estime de h  l'issue de la procdure itrative
%   flag : indicateur de convergence
%          flag = 0 : le seuil d'arrt a t atteint en moins de
%                     maxiter itrations
%          flag = 1 : maxiter itrations ont t effectues sans que
%                     le seuil d'arrt puisse tre atteint
%   iter : nombre d'itrations effectues
%   resvec : valeur de la norme relative du gradient  chaque
%            itration
%

eval(['afun = @(h) ', funQh, '(h, par);']);
[hest, flagcgs, ~, iter, resvec] = cgs(afun, b, tol, maxiter);

resvec = resvec(2:end) / norm(b);

flag = 0;
if flagcgs >= 1
    flag = 1;
end

fprintf('\ngradient_conjugue a termin aprs %1.0f itrations\n', iter)
fprintf('Norme relative des rsidus : %5.3g\n', resvec(iter))

end
